#include <stdio.h>
#include <stdlib.h>
#include <vga.h>
#include <vgakeyboard.h>
#include <pthread.h>
#include <sched.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <signal.h>
#include "CharMap.h"
#include "cardfeed.h"
#include "input.h"
#define INIT_GLOBALS
#include "loopdata.h"
#include "gui.h"
#include "audio.h"

extern void fx_init(); // from fxdata.h
extern void fx_initmutex(); // from fxdata.h
extern void init_Eff_Internals(); // from pump.h

static int APID = 0;
static int GPID = 0;

#define HZ 44100
#define BITS 16
#define STEREO 2
#define BUF_FRAGS 5
static int BUFFER_SIZE = 256;

#define NX 640
#define NY 480
/* initialize palette */
typedef struct _palentry
{
	int R;
	int G;
	int B;
} palentry;

palentry palette[16]=
{
{0, 0, 0},
{0, 2, 20},
{0, 3, 34},
{0, 0, 47},
{0, 0, 63},
{21, 19, 21},
{19, 23, 30},
{27, 29, 28},
{19, 63, 4},
{63, 63, 12},
{63, 29, 0},
{63, 0, 0},
{26, 0, 21},
{36, 0, 31},
{63, 0, 63},
{63, 63, 63}
};
volatile int rt_die = 0;
volatile unsigned int ticks = 0;
void * test_thread_start (void * P);
pthread_t test_thread;

int DISPLAY_SCREEN = 1;
int NEW_DISPLAY_SCREEN = 1;

void kaput (int sig)
{
	int pid;
	vga_setmode(TEXT);
	printf("\nSignal caught: ");
	switch (sig)
	{
	case SIGABRT:	printf("SIGABORT"); break;
	case SIGFPE:	printf("SIGFPE"); break;
	case SIGILL:	printf("SIGILL"); break;
	case SIGSEGV:	printf("SIGSEGV"); break;
	}
	printf(" in ");
	pid = getpid();
	printf("PID = %d\n", pid);
	printf("GUI: %d AUDIO: %d\n", GPID, APID);
	// in case this is only the GUI thread dying //
	// not really sure how signals work with threads //
	rt_die = 1; 
	exit(1);
}

int zeds_dead_honey = 0;

int main(int argc, char *argv[])
{
   int x;
   // get command line parameter for buffer size - should be a digit between 0 and 5 //
   if (argc > 1)
   {
   	x = (*argv[1])-'0';
	if (x >= 0 && x <= 5) BUFFER_SIZE = fs[x];
	printf("buffer size = %d\n", BUFFER_SIZE);
   }
   
   save_terminal_settings();
   init_l_parms(); // set up globals for loop screen //
   fx_init();
   init_Eff_Internals();
   GPID = getpid();
   // split off thread //
/*
   signal (SIGABRT, kaput);
   signal (SIGFPE, kaput);
   signal (SIGILL, kaput);
   signal (SIGSEGV, kaput);
*/
   pthread_create(&test_thread, NULL, test_thread_start, 0);
   init_input();
   if (vga_init() != 0)
   {
   	fprintf(stderr, "Couldn't initialize VGA graphics console, quitting.\n");
	exit(1);
   }
   

   vga_setmode(G640x480x16);
   vga_setcolor(0);
   // set these now - vga calls wipe signal handlers.
/*
   signal (SIGABRT, kaput);
   signal (SIGFPE, kaput);
   signal (SIGILL, kaput);
   signal (SIGSEGV, kaput);
*/
   /* load palette in */
   for (x=0;x<16;x++)
   {
   	vga_setpalette(x,palette[x].R, palette[x].G, palette[x].B);
   }
   draw_static_screen1();
   scan_keys();
   scan_mouse();   
   while (!is_pushed(SCANCODE_ESCAPE) || 
   		(DISPLAY_SCREEN == 4 || DISPLAY_SCREEN == 2) )
   {
	if (zeds_dead_honey) break;
	switch (DISPLAY_SCREEN)
	{
	default: case 1:
		handle_keyboard_screen1();
		draw_vu_bar(left_vu, 5, 0, 1);
		draw_vu_bar(right_vu, 59, 0, -1);
		draw_bpm_meter(get_bpm(),ticks,32,0);
		refresh_mouse_spinners();
		refresh_parm_boxes_scr1();
		update_efx_parms_scr3();
		break;
	case 2:
		handle_keyboard_screen2();
		break;
	case 3:
		handle_keyboard_screen3();
		draw_vu_bar(left_vu, 5, 0, 1);
		draw_vu_bar(right_vu, 59, 0, -1);
		draw_bpm_meter(get_bpm(),ticks,32,0);
		refresh_mouse_spinners();
		update_parm_boxes_scr1();
		update_efx_parms_scr3();
		refresh_efx_parms_scr3();
		break;
	case 4:
		handle_keyboard_screen4();
		break;
	};
	// screens switched? //
	if (NEW_DISPLAY_SCREEN != DISPLAY_SCREEN) 
	{
		vga_clear();
		DISPLAY_SCREEN = NEW_DISPLAY_SCREEN;
		switch (DISPLAY_SCREEN)
		{
		case 1: draw_static_screen1(); break;
		case 2: draw_static_screen2(); break;
		case 3: draw_static_screen3(); break;
		case 4: draw_static_screen4(); break;
		};
	}
	vga_waitretrace();
   	scan_keys();
	scan_mouse();
   }

   vga_setmode(TEXT);
   close_input();
   rt_die = 1;
   //while (rt_die);
   restore_terminal_settings();
   destroyLoopMutexes();
   exit(0);
}

void audio_thread_dead()
{
	zeds_dead_honey = 1;
	fprintf(stderr, "audio thread exited.  ELE is quitting.\n");
	pthread_exit(0);
}


void * test_thread_start (void * P)
{
	struct sched_param schp;
        memset(&schp, 0, sizeof(schp));
        schp.sched_priority = sched_get_priority_max(SCHED_FIFO);
        if (sched_setscheduler(0,SCHED_FIFO, &schp) != 0)
		printf("Can't get realtime priority for audio thread\n");
	APID = getpid();
	if (open_sound(HZ, BITS, STEREO, BUF_FRAGS, BUFFER_SIZE))
	{
		fprintf(stderr, "couldn't open audio device.\n");
		audio_thread_dead();
	}
	init_effect_buffers();
	fx_initmutex();
	while (!rt_die)
	{
		if (0 != read_sound() )
			{printf("Error reading sound - quitting.\n"); exit(0);}
		ticks += buf_size / 4;
		record_loopers();
		run_looper();
		run_effects();
		patch_effect_output();
		mixdown_looper();
		write_sound();
	
	}
	rt_die = 0;	
	return 0;
}


